@@ -11,10 +11,13 @@ import sys
 from pathlib import Path
 import os
 import collections
+import datetime
 
-version = __version__ = "1.1.11"
+version = __version__ = "1.1.13"
 mysimplegui_version = version
 
+pysimplegui_name = "PySimpleGUI"
+pysimplegui_patched_name = pysimplegui_name + "_patched"
 
 class peekable:
     def __init__(self, iterable):
@@ -37,7 +40,7 @@ class peekable:
 
 
 registered_patches = collections.defaultdict(int)
-
+registered_patches[37] = 0
 
 def register_patch(n):
     registered_patches[n] += 1
@@ -73,62 +76,92 @@ class CodeList(list):
         for line in lines:
             self.append(indentstr + line.replace("\x1b", "\\x1b"))
 
-
-pysimplegui_name = "PySimpleGUI"
-write_filename = pysimplegui_name + "_patched.py"
-write_filename = str(Path(write_filename).resolve())
-
 for path in sys.path:
-    source = Path(path) / (pysimplegui_name + ".py")
-    if source.is_file():
+    pysimplegui_path = Path(path) / (pysimplegui_name + ".py")
+    if pysimplegui_path.is_file():
         break
-    source = Path(path) / pysimplegui_name / (pysimplegui_name + ".py")
-    if source.is_file():
+    pysimplegui_path = Path(path) / pysimplegui_name / (pysimplegui_name + ".py")
+    if pysimplegui_path.is_file():
         break
 else:
     raise ImportError(pysimplegui_name + ".py" + " not found")
 
+mysimplegui_path = Path(__file__)
 
-with open(source, "r", encoding="utf-8") as f:
-    lines = peekable(f.read().splitlines())
+app_path = Path(sys.modules["__main__"].__file__)
+pysimplegui_patched_path = app_path.parent / (pysimplegui_patched_name + ".py")
 
-code = CodeList()
-while lines.peek().startswith("#!"):
-    code.add(next(lines))
+patch_info = "# patch info "
+stat = os.stat(pysimplegui_path)
+patch_info += "{st_mtime:16.3f}{st_size:8d}".format(st_mtime=stat.st_mtime, st_size=stat.st_size)
+stat = os.stat(mysimplegui_path)
+patch_info += "#{st_mtime:16.3f}{st_size:8d}".format(st_mtime=stat.st_mtime, st_size=stat.st_size)
 
-code.add(
-    """\
+pysimplegui_patched_match = False
+if pysimplegui_patched_path.is_file():
+    try:
+        with open(pysimplegui_patched_path, "rb") as f:
+            f.seek(-len(patch_info),os.SEEK_END)
+            read_patch_info = f.read().decode('utf-8')
+            if read_patch_info == patch_info:
+                pysimplegui_patched_match = True
+    except OSError:
+        pass
 
+if not pysimplegui_patched_match:
+    with open(pysimplegui_path, "r", encoding="utf-8") as f:
+        lines = peekable(f.read().splitlines())
+
+    code = CodeList()
+    while lines.peek().startswith("#!"):
+        code.add(next(lines))
+
+    code.add(
+        """\
 # This is {pysimplegui_name}.py patched by MySimpleGUI version {version}
 # Patches (c)2020  Ruud van der Ham, salabim.org
 
 """.format(
-        version=version, pysimplegui_name=pysimplegui_name
+            version=version, pysimplegui_name=pysimplegui_name
+        )
     )
-)
 
-this_class = "?"
-for line in lines:
-    if line.startswith("class "):
-        parts = line.split()
-        if len(parts) > 1:
-            this_class = parts[1].split("(")[0].replace(":", "").strip()
+    this_class = "?" 
+    for line in lines:
+        if "is True" in line and line.strip()[0] != ":":
+            s = line.split("is True", 1)[0]
+            s1 = s.split()[-1]
+            if s1 + " is True" not in line:
+                register_patch(37)
+            line = line.replace(s1 + " is True", "bool(" + s1 + ")")
 
-    if line.strip().startswith("self.config_count = 0"):
-        register_patch(0)
-        code.add(line)
-        indent = line_to_indent(line)
-        code.add(
-            """\
+        if "is False" in line and line.strip()[0] != ":":
+            s = line.split("is False", 1)[0]
+            s1 = s.split()[-1]
+            if s1 + " is False" not in line:
+                register_patch(37)
+            line = line.replace(s1 + " is False", "(not " + s1 + " and " + s1 + " is not None)")
+
+        if line.startswith("class "):
+            parts = line.split()
+            if len(parts) > 1:
+                this_class = parts[1].split("(")[0].replace(":", "").strip()
+
+        if line.strip().startswith("self.config_count = 0"):
+            register_patch(0)
+            code.add(line)
+            indent = line_to_indent(line)
+            code.add(
+                """\
 self.internal = Window.Internal(self)""",
             indent=indent,
-        )
+            )
 
-    elif "def Read(self, timeout=None, timeout_key=TIMEOUT_KEY, close=False):" in line:  # adds attributes to Read
-        register_patch(1)
-        indent = line_to_indent(line)
-        code.add(
-            """\
+        elif "def Read(self, timeout=None, timeout_key=TIMEOUT_KEY, close=False):" in line:  # adds attributes to Read
+            register_patch(1)
+            indent = line_to_indent(line)
+            code.add(
+                """\
 @staticmethod
 def _lookup(d, key):
     try:
@@ -209,16 +242,16 @@ def __getattr__(self, key):
     except KeyError as e:
         raise AttributeError(e) from None
 """,
-            indent=indent,
-        )
+                indent=indent,
+            )
 
-        code.add(line)
-        while not lines.peek().strip().startswith("return results"):
-            code.add(next(lines))
-        line = next(lines)
-        indent = line_to_indent(line)
-        code.add(
-            """\
+            code.add(line)
+            while not lines.peek().strip().startswith("return results"):
+                code.add(next(lines))
+            line = next(lines)
+            indent = line_to_indent(line)
+            code.add(
+                """\
 class AttributeDict(collections.UserDict):
     def __getitem__(self, key):
         return Window._lookup(self.data, key)
@@ -236,15 +269,15 @@ if values is not None:
     values = AttributeDict(values)
 results = events, values
 """,
-            indent=indent,
-        )
-        code.add(line)  # the original return result line
+                indent=indent,
+            )
+            code.add(line)  # the original return result line
 
-    elif line.strip().startswith("def _RightClickMenuCallback(self, event):"):
-        register_patch(2)
-        indent = line_to_indent(line)
-        code.add(
-            """\
+        elif line.strip().startswith("def _RightClickMenuCallback(self, event):"):
+            register_patch(2)
+            indent = line_to_indent(line)
+            code.add(
+                """\
 def type_name(self):
     type_name = self.Type
     type_name = type_name.replace("option menu","OptionMenu")
@@ -278,15 +311,22 @@ def __repr__(self):
                         for x in l[1:]:
                             result.append("    " + x)
         del self.taken
-    return "\\n".join(result)""",
-            indent=indent,
-        )
+    return "\\n".join(result)
+    
 
-        code.add(line)
-    elif line.startswith("class Element"):
-        register_patch(3)
-        code.add(
-            """\
+def freeze(self, enabled=True):
+    if enabled:
+        return Column([[self]], pad=(0,0))
+    else:
+        return self""",
+                indent=indent,
+            )
+
+            code.add(line)
+        elif line.startswith("class Element"):
+            register_patch(3)
+            code.add(
+                """\
 class list_repr(list):
     def type_name(self):
         return "list"
@@ -301,71 +341,71 @@ class list_repr(list):
             for x in l:
                 result.append("    " + x)
         return "\\n".join(result)"""
-        )
+            )
 
-        code.add(line)
-    elif line.strip().startswith("return self.FindElement(key)"):  # this the original Window.__getitem__ contents:
-        register_patch(4)
-        code.add(line.replace("return self.FindElement(key)", "return Window._lookup(self.AllKeysDict, key)"))
+            code.add(line)
+        elif line.strip().startswith("return self.FindElement(key)"):  # this the original Window.__getitem__ contents:
+            register_patch(4)
+            code.add(line.replace("return self.FindElement(key)", "return Window._lookup(self.AllKeysDict, key)"))
 
-    elif line.strip().startswith("if file_name"):
-        register_patch(5)
-        code.add(line.replace("file_name", "file_name or self is target_element"))
+        elif line.strip().startswith("if file_name"):
+            register_patch(5)
+            code.add(line.replace("file_name", "file_name or self is target_element"))
 
-    elif line.strip() == "if folder_name:":
-        register_patch(6)
-        code.add(line.replace("folder_name", "True"))
+        elif line.strip() == "if folder_name:":
+            register_patch(6)
+            code.add(line.replace("folder_name", "True"))
 
-    elif "target=target" in line:
-        register_patch(7)
-        code.add(line.replace("target=target", "target=target or key or k or button_text"))
+        elif "target=target" in line:
+            register_patch(7)
+            code.add(line.replace("target=target", "target=target or key or k or button_text"))
 
-    elif line.strip().startswith("if element.Type == ELEM_TYPE_INPUT_TEXT:"):
-        register_patch(8)
-        code.add(line)
-        if lines.peek().strip().startswith("try:"):
-            while True:
-                line = next(lines)
-                if line.strip().startswith("elif"):
-                    break
+        elif line.strip().startswith("if element.Type == ELEM_TYPE_INPUT_TEXT:"):
+            register_patch(8)
+            code.add(line)
+            if lines.peek().strip().startswith("try:"):
+                while True:
+                    line = next(lines)
+                    if line.strip().startswith("elif"):
+                        break
+                    code.add(line)
+                indent = line_to_indent(line)
+                code.add(
+                    """\
+elif element.Type == ELEM_TYPE_TEXT:
+    value = element.TKStringVar.get()""",
+                    indent=indent,
+                )
                 code.add(line)
+
+        elif "ELEM_TYPE_SEPARATOR):" in line:
+            register_patch(9)
+            code.add(line.replace("ELEM_TYPE_SEPARATOR", "ELEM_TYPE_SEPARATOR, ELEM_TYPE_TEXT"))
+
+        elif "element.Type != ELEM_TYPE_TEXT and \\" in line:
+            register_patch(10)
+            pass  # remove this condition
+
+        elif "if element.Key in key_dict.keys():" in line:
+            register_patch(11)
+            #        code.add(line)
             indent = line_to_indent(line)
             code.add(
                 """\
-elif element.Type == ELEM_TYPE_TEXT:
-    value = element.TKStringVar.get()""",
-                indent=indent,
-            )
-            code.add(line)
-
-    elif "ELEM_TYPE_SEPARATOR):" in line:
-        register_patch(9)
-        code.add(line.replace("ELEM_TYPE_SEPARATOR", "ELEM_TYPE_SEPARATOR, ELEM_TYPE_TEXT"))
-
-    elif "element.Type != ELEM_TYPE_TEXT and \\" in line:
-        register_patch(10)
-        pass  # remove this condition
-
-    elif "if element.Key in key_dict.keys():" in line:
-        register_patch(11)
-        #        code.add(line)
-        indent = line_to_indent(line)
-        code.add(
-            """\
 if element.Key == "AllKeysDictxx":
     raise KeyError('key "AllKeysDict" not allowed in layout')""",
-            indent=indent,
-        )
+                indent=indent,
+            )
 
-        while line_to_indent(lines.peek()) > indent:  # remove original code
-            next(lines)
+            while line_to_indent(lines.peek()) > indent:  # remove original code
+                next(lines)
 
-    elif "self.AllKeysDict = self._BuildKeyDictForWindow(self, self, dict)" in line:
-        register_patch(34)
-        indent = line_to_indent(line)
-        code.add(line)
-        code.add(
-            """\
+        elif "self.AllKeysDict = self._BuildKeyDictForWindow(self, self, dict)" in line:
+            register_patch(34)
+            indent = line_to_indent(line)
+            code.add(line)
+            code.add(
+                """\
 self.AllKeysDictnewdict = {}
 for k, v in dict.items():
     if not(isinstance(k, tuple) and len(k) == 2 and k[0] == NONE_KEY):
@@ -378,43 +418,43 @@ for k, v in dict.items():
                 v.Key = i
                 self.AllKeysDictnewdict[k] = v
                 break""",
-            indent=indent,
-        )
+                indent=indent,
+            )
 
-    elif "element.Key = top_window.DictionaryKeyCounter" in line:
-        register_patch(35)
-        indent = line_to_indent(line)
-        code.add("element.Key = (NONE_KEY, top_window.DictionaryKeyCounter)", indent=indent)
+        elif "element.Key = top_window.DictionaryKeyCounter" in line:
+            register_patch(35)
+            indent = line_to_indent(line)
+            code.add("element.Key = (NONE_KEY, top_window.DictionaryKeyCounter)", indent=indent)
 
-    elif line.startswith("COLOR_SYSTEM_DEFAULT = "):
-        register_patch(36)
-        code.add("NONE_KEY = object()")
-        code.add(line)
+        elif line.startswith("COLOR_SYSTEM_DEFAULT = "):
+            register_patch(36)
+            code.add("NONE_KEY = object()")
+            code.add(line)
 
-    elif line == "class Multiline(Element):":
-        register_patch(12)
-        code.add(line)
-        while not lines.peek().strip().startswith("self."):
-            code.add(next(lines))
-        indent = line_to_indent(lines.peek())
-        code.add(
-            """\
+        elif line == "class Multiline(Element):":
+            register_patch(12)
+            code.add(line)
+            while not lines.peek().strip().startswith("self."):
+                code.add(next(lines))
+            indent = line_to_indent(lines.peek())
+            code.add(
+                """\
 self._closed = False
 self.write_fg = None
 self.write_bg = None
 self.write_font = None""",
-            indent=indent,
-        )
+                indent=indent,
+            )
 
-    elif "def write(self, txt):" in line and this_class == "Multiline":
-        register_patch(13)
+        elif "def write(self, txt):" in line and this_class == "Multiline":
+            register_patch(13)
 
-        indent = line_to_indent(line)
-        while line_to_indent(lines.peek()) > indent:
-            next(lines)
+            indent = line_to_indent(line)
+            while line_to_indent(lines.peek()) > indent:
+                next(lines)
 
-        code.add(
-            """\
+            code.add(
+                """\
 class AttributeDict(collections.UserDict):
     def __getitem__(self, key):
         return self.data[key]
@@ -511,103 +551,103 @@ def _check_closed(self):
 
 def writable(self):
     return not self._closed""",
-            indent=indent,
-        )
-
-    elif "element.TKText.insert(1.0, element.DefaultText)" in line:
-        register_patch(14)
-        code.add(
-            line.replace("element.TKText.insert(1.0, element.DefaultText)", "print(element.DefaultText, file=element)")
-        )
-
-    elif "background_color_for_value=background_color" in line:
-        register_patch(15)
-        code.add(
-            line.replace(
-                "background_color_for_value=background_color",
-                "background_color_for_value=background_color, font_for_value=font",
+                indent=indent,
             )
-        )
 
-    elif "background_color_for_value=None" in line:
-        register_patch(16)
-        code.add(
-            line.replace("background_color_for_value=None", "background_color_for_value=None, font_for_value=None")
-        )
-
-    elif "if background_color_for_value is not None or text_color_for_value is not None:" in line:
-        register_patch(17)
-        code.add(line.replace(":", " or font_for_value is not None:"))
-
-    elif "str(background_color_for_value)+')'" in line:
-        register_patch(18)
-        code.add(
-            line.replace(
-                "str(background_color_for_value)+')'", "str(background_color_for_value)+',' + str(font_for_value)+')'"
+        elif "element.TKText.insert(1.0, element.DefaultText)" in line:
+            register_patch(14)
+            code.add(
+                line.replace("element.TKText.insert(1.0, element.DefaultText)", "print(element.DefaultText, file=element)")
             )
-        )
 
-    elif "text_color=None, background_color=None" in line and line.strip().startswith(
-        ("def Print", "def print", "def _print_to_element")
-    ):
-        register_patch(19)
-        code.add(
-            line.replace("text_color=None, background_color=None", "text_color=None, background_color=None, font=None")
-        )
-
-    elif "text_color=text_color, background_color=background_color" in line and line.strip().startswith(
-        "_print_to_element"
-    ):
-        register_patch(20)
-        code.add(
-            line.replace(
-                "text_color=text_color, background_color=background_color",
-                "text_color=text_color, background_color=background_color, font=font",
+        elif "background_color_for_value=background_color" in line:
+            register_patch(15)
+            code.add(
+                line.replace(
+                    "background_color_for_value=background_color",
+                    "background_color_for_value=background_color, font_for_value=font",
+                )
             )
-        )
 
-    elif "if background_color_for_value is not None:" in line:
-        register_patch(21)
-        indent = line_to_indent(line)
-        code.add(
-            """\
+        elif "background_color_for_value=None" in line:
+            register_patch(16)
+            code.add(
+                line.replace("background_color_for_value=None", "background_color_for_value=None, font_for_value=None")
+            )
+
+        elif "if background_color_for_value is not None or text_color_for_value is not None:" in line:
+            register_patch(17)
+            code.add(line.replace(":", " or font_for_value is not None:"))
+
+        elif "str(background_color_for_value)+')'" in line:
+            register_patch(18)
+            code.add(
+                line.replace(
+                    "str(background_color_for_value)+')'", "str(background_color_for_value)+',' + str(font_for_value)+')'"
+                )
+            )
+
+        elif "text_color=None, background_color=None" in line and line.strip().startswith(
+            ("def Print", "def print", "def _print_to_element")
+        ):
+            register_patch(19)
+            code.add(
+                line.replace("text_color=None, background_color=None", "text_color=None, background_color=None, font=None")
+            )
+
+        elif "text_color=text_color, background_color=background_color" in line and line.strip().startswith(
+            "_print_to_element"
+        ):
+            register_patch(20)
+            code.add(
+                line.replace(
+                    "text_color=text_color, background_color=background_color",
+                    "text_color=text_color, background_color=background_color, font=font",
+                )
+            )
+
+        elif "if background_color_for_value is not None:" in line:
+            register_patch(21)
+            indent = line_to_indent(line)
+            code.add(
+                """\
 if font_for_value is not None:
     self.TKText.tag_configure(tag, font=font_for_value)""",
-            indent=indent,
-        )
-        code.add(line)
-
-    elif "if element.Key is not None:" in line:
-        register_patch(22)
-        code.add(
-            line.replace(
-                "if element.Key is not None:",
-                "if element.Key is not None and isinstance(element.Key, collections.abc.Hashable):",
+                indent=indent,
             )
-        )
+            code.add(line)
 
-    elif line.strip().startswith("form.ReturnValuesDictionary[element.Key] = value"):
-        register_patch(23)
-        indent = line_to_indent(line)
-        code.add(
-            """\
+        elif "if element.Key is not None:" in line:
+            register_patch(22)
+            code.add(
+                line.replace(
+                    "if element.Key is not None:",
+                    "if element.Key is not None and isinstance(element.Key, collections.abc.Hashable):",
+                )
+            )
+
+        elif line.strip().startswith("form.ReturnValuesDictionary[element.Key] = value"):
+            register_patch(23)
+            indent = line_to_indent(line)
+            code.add(
+                """\
 if isinstance(element.Key, collections.abc.Hashable):
     form.ReturnValuesDictionary[element.Key] = value""",
-            indent=indent,
-        )
+                indent=indent,
+            )
 
-    elif line.startswith("def SetOptions("):  # generates extra function to get/set globals
-        register_patch(24)
-        names = {}
-        code.add(line)
-        while line_to_indent(lines.peek()) > 0:
-            line = next(lines)
+        elif line.startswith("def SetOptions("):  # generates extra function to get/set globals
+            register_patch(24)
+            names = {}
             code.add(line)
-            left, *right = line.split(" = ")
-            if "#" not in line and "." not in line and line.startswith("        ") and right:
-                names[left.strip()] = right[0]
-        code.add(
-            """\
+            while line_to_indent(lines.peek()) > 0:
+                line = next(lines)
+                code.add(line)
+                left, *right = line.split(" = ")
+                if "#" not in line and "." not in line and line.startswith("        ") and right:
+                    names[left.strip()] = right[0]
+            code.add(
+                """\
 class _TemporaryChange:
     def __init__(self, value, global_name, globals):
         self.globals=globals
@@ -620,146 +660,145 @@ class _TemporaryChange:
         self.globals[self.global_name] = self.save_value
     def __call__(self):
         return self.globals[self.global_name]"""
-        )
+            )
 
-        names["RAISE_ERRORS"] = "raise_errors"
-        for name in sorted(names, key=lambda x: names[x]):
-            alias = names[name]
-            code.add(
-                """\
+            names["RAISE_ERRORS"] = "raise_errors"
+            for name in sorted(names, key=lambda x: names[x]):
+                alias = names[name]
+                code.add(
+                    """\
 def {alias}(value = None):
     global {name}
     if value is None:
         return {name}
     return _TemporaryChange(value, '{name}', globals())""".format(
-                    name=name, alias=alias
+                        name=name, alias=alias
+                    )
                 )
-            )
 
-    elif line.startswith("def PopupError("):  # changes PopupError behaviour to raise exception
-        register_patch(25)
-        indent = line_to_indent(line)
+        elif line.startswith("def PopupError("):  # changes PopupError behaviour to raise exception
+            register_patch(25)
+            indent = line_to_indent(line)
 
-        code.add(line)
-        while ":" not in lines.peek():  # read till final :
+            code.add(line)
+            while ":" not in lines.peek():  # read till final :
+                code.add(next(lines))
             code.add(next(lines))
-        code.add(next(lines))
 
-        code.add(
-            """
+            code.add(
+                """
 trace_details = traceback.format_stack()
 if (trace_details[-1].split(",")[0] == trace_details[-2].split(",")[0]) and RAISE_ERRORS:
     raise RuntimeError("\\n".join(args))""",
-            indent=indent + 4,
-        )
+                indent=indent + 4,
+            )
 
-    elif line == "import queue":  # add some required modules
-        register_patch(26)
-        code.add(line)
-        code.add(
-            """\
+        elif line == "import queue":  # add some required modules
+            register_patch(26)
+            code.add(line)
+            code.add(
+                """\
 import keyword
 import collections
 import io"""
-        )
-
-    elif line.startswith("version = "):  # check compatibilty of PySimpleGUI version (at patch time)
-        register_patch(27)
-        code.add(line)
-        exec(line)
-        this_pysimplegui_version = version.split()[0]
-        minimal_pysimplegui_version = "4.27.4"
-        this_pysimplegui_version_tuple = tuple(map(int, this_pysimplegui_version.split(".")))
-        minimal_pysimplegui_version_tuple = tuple(map(int, minimal_pysimplegui_version.split(".")))
-
-        if this_pysimplegui_version_tuple < minimal_pysimplegui_version_tuple:
-            raise NotImplementedError(
-                "MySimpleGUI requires "
-                + pysimplegui_name
-                + " >= "
-                + minimal_pysimplegui_version
-                + ", not "
-                + this_pysimplegui_version
             )
-        del version
-        del this_pysimplegui_version
-        del minimal_pysimplegui_version
 
-    elif line.strip() in ("except:", "except Exception as e:"):  # exception handler insertion
-        register_patch(28)
-        indent = line_to_indent(line)
-        exception_line = line
-        buffered_lines = []
-        while line_to_indent(lines.peek()) > indent:
-            line = next(lines)
-            if line.strip().startswith("warnings.warn"):
-                line = line.replace("warnings.warn", "print")
-            buffered_lines.append(line)
-        requires_raise = False
-        for line in buffered_lines:
-            if line.strip().startswith("pass") or line.strip().startswith("return") or line.strip().startswith("break"):
-                requires_raise = False
-                break
-            if line.strip().startswith("print("):
-                requires_raise = True
-        if requires_raise:
-            code.add("except Exception as e:", indent=indent)
-            code.add(
-                """\
+        elif line.startswith("version = "):  # check compatibilty of PySimpleGUI version (at patch time)
+            register_patch(27)
+            code.add(line)
+            exec(line)
+            this_pysimplegui_version = version.split()[0]
+            minimal_pysimplegui_version = "4.27.4"
+            this_pysimplegui_version_tuple = tuple(map(int, this_pysimplegui_version.split(".")))
+            minimal_pysimplegui_version_tuple = tuple(map(int, minimal_pysimplegui_version.split(".")))
+
+            if this_pysimplegui_version_tuple < minimal_pysimplegui_version_tuple:
+                raise NotImplementedError(
+                    "MySimpleGUI requires "
+                    + pysimplegui_name
+                    + " >= "
+                    + minimal_pysimplegui_version
+                    + ", not "
+                    + this_pysimplegui_version
+                )
+            del version
+            del this_pysimplegui_version
+            del minimal_pysimplegui_version
+
+        elif line.strip() in ("except:", "except Exception as e:"):  # exception handler insertion
+            indent = line_to_indent(line)
+            exception_line = line
+            buffered_lines = []
+            while line_to_indent(lines.peek()) > indent:
+                line = next(lines)
+                if line.strip().startswith("warnings.warn"):
+                    line = line.replace("warnings.warn", "print")
+                buffered_lines.append(line)
+            requires_raise = False
+            for line in buffered_lines:
+                if line.strip().startswith("pass") or line.strip().startswith("return") or line.strip().startswith("break"):
+                    requires_raise = False
+                    break
+                if line.strip().startswith("print("):
+                    requires_raise = True
+            if requires_raise:
+                code.add("except Exception as e:", indent=indent)
+                code.add(
+                    """\
 if RAISE_ERRORS:
     save_stdout = sys.stdout
     sys.stdout = io.StringIO()""",
-                indent=indent + 4,
-            )
-            code.add(buffered_lines)
+                    indent=indent + 4,
+                )
+                code.add(buffered_lines)
 
-            code.add(
-                """\
+                code.add(
+                    """\
 if RAISE_ERRORS:
     captured = sys.stdout.getvalue()
     sys.stdout = save_stdout
     if captured:
         raise type(e)(str(e) + '\\n' + captured) from None""",
-                indent=indent + 4,
-            )
-        else:
-            code.add(exception_line)
-            code.add(buffered_lines)
+                    indent=indent + 4,
+                )
+            else:
+                code.add(exception_line)
+                code.add(buffered_lines)
 
-    #    elif line.strip().startswith("if key in settings.dict:"):
-    #        indent = line_to_indent(line)
-    #        while not lines.peek().strip().startswith("else:"):
-    #            code.add(next(lines).strip(), indent=indent)
-    #        while line_to_indent(next(lines)) == indent:
-    #            next(lines)
+        #    elif line.strip().startswith("if key in settings.dict:"):
+        #        indent = line_to_indent(line)
+        #        while not lines.peek().strip().startswith("else:"):
+        #            code.add(next(lines).strip(), indent=indent)
+        #        while line_to_indent(next(lines)) == indent:
+        #            next(lines)
 
-    elif "SUPPRESS_ERROR_POPUPS = False" in line:  # set global RAISE_ERRORS
-        register_patch(29)
-        code.add(line)
-        indent = line_to_indent(line)
-        code.add("RAISE_ERRORS = True", indent=indent)
-
-    elif "ix = random.randint(0, len(lf_values) - 1)" in line:  # no more random theme selection, but exception
-        register_patch(30)
-        indent = line_to_indent(line)
-        code.add("raise ValueError(index + ' not a valid theme')", indent=indent)
-        while line_to_indent(lines.peek()) >= indent:
-            next(lines)
-
-    elif line.strip().startswith("warnings.warn("):
-        register_patch(31)
-        if "popup" in lines.peek().lower() and "error" in lines.peek().lower():
-            pass  # remove line
-        else:
-            line = line.replace("warnings.warn", "raise RuntimeError")
-            line = line.replace(", UserWarning", "")
+        elif "SUPPRESS_ERROR_POPUPS = False" in line:  # set global RAISE_ERRORS
+            register_patch(29)
             code.add(line)
+            indent = line_to_indent(line)
+            code.add("RAISE_ERRORS = True", indent=indent)
 
-    elif "photo = tk.PhotoImage(file=element.Filename)" in line:
-        register_patch(32)
-        indent = line_to_indent(line)
-        code.add(
-            """\
+        elif "ix = random.randint(0, len(lf_values) - 1)" in line:  # no more random theme selection, but exception
+            register_patch(30)
+            indent = line_to_indent(line)
+            code.add("raise ValueError(index + ' not a valid theme')", indent=indent)
+            while line_to_indent(lines.peek()) >= indent:
+                next(lines)
+
+        elif line.strip().startswith("warnings.warn("):
+            register_patch(31)
+            if "popup" in lines.peek().lower() and "error" in lines.peek().lower():
+                pass  # remove line
+            else:
+                line = line.replace("warnings.warn", "raise RuntimeError")
+                line = line.replace(", UserWarning", "")
+                code.add(line)
+
+        elif "photo = tk.PhotoImage(file=element.Filename)" in line:
+            register_patch(32)
+            indent = line_to_indent(line)
+            code.add(
+                """\
 try:
     import PIL
     from PIL import ImageTk
@@ -784,14 +823,14 @@ else:
     else:
         raise ValueError("PIL image format not supported. Try installing PIL")
 """,
-            indent=indent,
-        )
+                indent=indent,
+            )
 
-    elif "image = tk.PhotoImage(file=filename)" in line:
-        register_patch(33)
-        indent = line_to_indent(line)
-        code.add(
-            """\
+        elif "image = tk.PhotoImage(file=filename)" in line:
+            register_patch(33)
+            indent = line_to_indent(line)
+            code.add(
+                """\
 try:
     import PIL
     from PIL import ImageTk
@@ -817,15 +856,15 @@ else:
         raise ValueError("PIL image format not supported. Try installing PIL")
 
 """,
-            indent=indent,
-        )
-    elif "__delitem__" in line:
-        register_patch(34)
-        code.add(line)
-        indent = line_to_indent(line)
-        while line_to_indent(lines.peek()) >= indent:
-            code.add(next(lines))
-        code.add("""\
+                indent=indent,
+            )
+        elif "__delitem__" in line:
+            register_patch(35)
+            code.add(line)
+            indent = line_to_indent(line)
+            while line_to_indent(lines.peek()) >= indent:
+                code.add(next(lines))
+            code.add("""\
 def __getattr__(self, item):
     if item in self.dict:
         return self[item]
@@ -842,54 +881,62 @@ def __setattr__(self, item, value):
 def __delattr__(self, item):
     del self[item]""", indent = indent)
 
-    elif line.startswith("if __name__ == "):  # no more PySimpleGUI startup screen
-        register_patch(35)
-        break
+        elif line.strip() == "return self.update(*args, **kwargs)":
+            register_patch(38)
+            code.add("return self.get()", indent=line_to_indent(line))
+
+        elif line.startswith("if __name__ == "):  # no more PySimpleGUI startup screen
+            register_patch(36)
+            break
+        else:
+            code.add(line)
+
+    code.add(patch_info)
+
+    to_be_registered_patches = {0: 1, 1: 1, 2: 1, 3: 1, 4: 1, 5: 3, 6: 1, 7: 7, 8: 1, 9: 1, 10: 1, 11: 1, 12: 1, 13: 1, 14: 1, 15: 2, 16: 1, 17: 1, 18: 1, 19: 3, 20: 1, 21: 1, 22: 6, 23: 1, 24: 1, 25: 1, 26: 1, 27: 1, 29: 1, 30: 1, 31: 9, 32: 1, 33: 2, 34: 1, 35: 2, 36: 2, 37:0, 38:1}
+    if 35 in registered_patches:
+        to_be_registered_patches[35] = registered_patches[35]  # ignore test for patch 35 as it may be 1 or 2
+    mismatch = False
+    for patch in to_be_registered_patches:
+        if patch in registered_patches:
+            if registered_patches[patch] != to_be_registered_patches[patch]:
+                print("patch {patch} should be applied {tbr} times , but it {r} times".format(patch=patch, tbr=to_be_registered_patches[patch], r=registered_patches[patch]))
+                mismatch = True
+        else:
+            print("patch {patch} should be applied {tbr} times, but is not registered at all".format(patch=patch, tbr=to_be_registered_patches[patch]))
+            mismatch=True
+
+    for patch in registered_patches:
+        if patch not in to_be_registered_patches:
+            print("patch {patch} is applied {r} times, but is not in the to_be_registered dict at all".format(patch=patch, r=registered_patches[patch]))
+            mismatch=True
+
+    if mismatch:
+        patches = {}
+        for patch in sorted(registered_patches):
+            patches[patch] = registered_patches[patch]
+        print("Suggested mod:")
+        print("to_be_registered_patches = {patches}".format(patches=patches))
 
 
-
-    else:
-        register_patch(34)
-        code.add(line)
-
-missing_patches = set(range(len(registered_patches))) - set(registered_patches)
-
-for var in list(vars().keys()):
-    if not var.startswith("__") and var not in (
-        "code",
-        "write_filename",
-        "os",
-        "mysimplegui_version",
-        "missing_patches",
-    ):
-        del vars()[var]
-
-
-def write_file():
-    with open(write_filename, "w", encoding="utf-8") as f:
+    if mismatch:
+        raise Warning("patches mismatch")
+    with open(pysimplegui_patched_path, "w", encoding="utf-8") as f:
         f.write("\n".join(code))
 
+for var in list(vars().keys()):
+    if var not in ("__name__", "mysimplegui_version", "pysimplegui_patched_path", "pysimplegui_patched_match"):
+        del vars()[var]
 
-# write_file()
+from PySimpleGUI_patched import *
 
-if "MySimpleGUI_full_traceback" in os.environ and os.environ["MySimpleGUI_full_traceback"] != "":
-    write_file()
-    from PySimpleGUI_patched import *
-else:
-    exec("\n".join(code))
-
-if missing_patches:
-    raise Warning("not all patches applied.\n" + "Missing: " + ", ".join(map(str, missing_patches)))
+__version__ = ver
 
 if __name__ == "__main__":
+    if pysimplegui_patched_match:
+        print(str(pysimplegui_patched_path) + " is up-to-date. Nothing written.")
+    else:
+        print(str(pysimplegui_patched_path) + " written successfully.")
 
-    with message_box_line_width(len(write_filename)):
-        if PopupYesNo("Would you like to save the patched version to\n" + write_filename) == "Yes":
-            write_file()
-            Popup("Patched version saved to\n" + write_filename, title="saved")
-
-del var
-del code
-del write_file
-del write_filename
-del missing_patches
+del pysimplegui_patched_path
+del pysimplegui_patched_match
